import os
import tempfile

from .application   import screenshots, resize, timestamps, thumb
from .viewer        import print_process, print_success

class Generator:
    def __init__(self, video_path, output_path='', rows=0, columns=0, imgCount=0, custom_text='True', font_dir='', font_size=0, bg_colour='', font_colour=''):
        self.video_path = video_path
        self.rows = rows
        self.columns = columns
        self.imgCount = imgCount

        if self.imgCount != 0:
            if self.rows == 0 and self.columns == 0:
                self.columns = 3
                self.rows, mod = divmod(self.imgCount, self.columns)
                if mod != 0 and self.imgCount > self.columns:
                    self.rows += 1

            elif self.rows == 0:
                if self.columns > self.imgCount:
                    raise ValueError("'columns' value greater than 'imgCount'.")
                self.rows, mod = divmod(self.imgCount, self.columns)
                if mod != 0 and self.imgCount > self.columns:
                    self.rows += 1

            elif self.columns == 0:
                if self.rows > self.imgCount:
                    raise ValueError("'rows' value greater than 'imgCount'.")
                self.columns, mod = divmod(self.imgCount, self.rows)
                if mod != 0 and self.imgCount > self.rows:
                    self.columns += 1

            else:
                raise TypeError("defining 'rows', 'columns' and 'imgCount' at once is not functional. remove one of them")
        else:
            if (self.rows == 0 and self.columns != 0) or (self.rows != 0 and self.columns == 0):
                raise ValueError("missing 1 required positional argument: 'imgCount'")

        if 0 < self.imgCount < 3:
            raise ValueError("'imgCount' value must be greater than 3")

        if 0 < self.columns < 3:
            raise ValueError("'columns' value must be greater than 3")

        if output_path == '':
            # better way toany extension length
            self.output_path = os.path.splitext(self.video_path)[0]
            self.output_folder = os.path.dirname(self.video_path)
        else:
            # Extract filename and strip extension
            filename = os.path.basename(self.video_path)
            filename_no_ext = os.path.splitext(filename)[0]
            
            self.output_path = os.path.join(output_path, filename_no_ext)
            self.output_folder = self.output_path

        self.custom_text = str(custom_text)
        self.font_dir = font_dir

        if isinstance(font_size, int):
            self.font_size = font_size
        elif isinstance(font_size, str):
            raise ValueError("'font_size' must be an integer")

        self.bg_colour = bg_colour
        self.font_colour = font_colour

        self.temp_dir = tempfile.TemporaryDirectory()
        self.secure_temp = self.temp_dir.name
        self.screenshot_folder = os.path.join(self.secure_temp, 'screenshots')
        self.resize_folder = os.path.join(self.secure_temp, 'resized')
        os.mkdir(self.screenshot_folder)
        os.mkdir(self.resize_folder)

    def run(self):
        print_process(self.video_path)
        self.ss_time = screenshots(self.video_path, self.screenshot_folder, self.imgCount)
        resize(self.screenshot_folder, self.resize_folder)
        timestamps(self.resize_folder, self.font_dir, self.font_size, self.ss_time)
        thumb_out = thumb(self.video_path,
                          self.rows,
                          self.columns,
                          self.output_path,
                          self.resize_folder,
                          self.secure_temp,
                          self.custom_text,
                          self.font_dir,
                          self.font_size,
                          self.bg_colour,
                          self.font_colour)

        if thumb_out:
            print_success(self.output_folder)

        return True


import os
import pytest

from thumb_gen import Generator

# Use local files so tests pass both locally and on GitHub Actions
video_path = "sample.mp4" 
output_path = os.getcwd()

def test_worker_1():
    # Checking whether it gives an error when 'columns' is given without 'imgCount'
    with pytest.raises(ValueError):
        Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", columns=10)

def test_worker_2():
    # Checking whether it gives an error when 'rows' is given without 'imgCount'
    with pytest.raises(ValueError):
        Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", rows=10)

def test_worker_3():
    # Checking if NO error is returned when required arguments are given
    app = Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", rows=10, columns=10)
    assert app is not None

def test_worker_4():
    # Checking if NO error is returned when required arguments are given
    app = Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", rows=10, imgCount=30)
    assert app is not None

def test_worker_5():
    # Checking if NO error is returned when required arguments are given
    app = Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", columns=10, imgCount=30)
    assert app is not None

def test_worker_6():
    # Check if an error is returned when image count is lower than rows
    with pytest.raises(ValueError):
        Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", rows=10, imgCount=5)

def test_worker_7():
    # Check if an error is returned when image count is lower than columns
    with pytest.raises(ValueError):
        Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", columns=10, imgCount=5)

def test_worker_8():
    # Check if an error is returned when defining 'rows', 'columns', and 'imgCount' all at once
    with pytest.raises(TypeError):
        Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", columns=10, rows=5, imgCount=50)

def test_worker_9():
    app = Generator(video_path, output_path=output_path, custom_text="thumb gen")
    assert app.run() == True

def test_worker_10():
    app = Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black")
    assert app.run() == True

def test_worker_11():
    app = Generator(video_path, output_path=output_path, font_size=10, bg_colour="yellow", font_colour="black", columns=5, rows=10)
    assert app.run() == True